#
#
set frametitle "MIDAS Data Acquisition System Control (TAS)"

set CAMACPresent 1
set VMEPresent 1

set CAMACEnable 1
set VMEEnable 1

if {[info exists env(CamacController)]} then {set resource $env(CamacController)} else {set resource CC}
set CAMACcontroller $resource

set TestBuffer "9010c723 d1658804 0c438801 4a5bffff"


#    list of menu items and associated actions for the ACTIONS menu button 

lappend actionlist [list "Enable HM413 task tracing"  "click-HM413trace 1"]
lappend actionlist [list "Disable HM413 task tracing" "click-HM413trace 0"]
lappend actionlist [list "Show HM413 task ID"         "click-HM413ID"]

lappend registerlist [list HM413.Sig1       sigTask  "-tHM413 -sUSR1"]
lappend registerlist [list HM413.Action     generic  "-nHM413_Action -tint"]
lappend registerlist [list HM413.ID         generic  "-nHM413_ID -tstring"]
lappend registerlist [list HM413.Trace      generic  "-nHM413_Trace -tint"]
lappend registerlist [list HM413.State      generic  "-nHM413_State -tint"]
lappend registerlist [list HM413.Crate      generic  "-nHM413_Crate -tint"]
lappend registerlist [list HM413.Station    generic  "-nHM413_Station -tint"]
lappend registerlist [list HM413.SegmentID  generic  "-nHM413_SegmentID -tint"]


set HM413_Segments 4
set HM413_SegmentLen [expr 32 * 1024 / $HM413_Segments]

set HM413_Present 0

proc Custom_PreCheck {arg} {
    global HM413_Present

    if {$arg == 2} then {set HM413_Present [inform camaccontrol inquire hm413count]}
}

proc create_C1011_spectra {module vsn} {
    global SpectrumServer HistMode

    DefineConfig "$vsn 7 1 0 0 0"

    if {$HistMode == 0} {return}

    for {set j 0} {$j <= 1} {incr j} {
      if {$j==0} then {set adc L} else {set adc H}

      set spectrumname [SpectrumName $module.$adc]

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 65536 :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
          clear-footer
          error $m
      } else {
          map_spectrum "$spectrumname $vsn $j"
 #insert-log "created $module.$adc for vsn=$vsn and sub address=$j"
      }
    }
}

proc create_hm_spectra {module vsn} {
   global SpectrumServer HistMode
   global HM413_Segments HM413_SegmentLen

#     hm413

    if {$HistMode == 0} {return}

    for {set j 0} {$j < $HM413_Segments} {incr j} {

      set spectrumname [SpectrumName $module.adc$j]

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $HM413_SegmentLen :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
          clear-footer
          error $m
      } else {
#          map_spectrum "$spectrumname $vsn $j"
 #insert-log "created $module.adc$j"
      }
    }

    map_HM413spectra
}

proc map_spectra {} {
   global RDOUTEnable

   if {$RDOUTEnable == 0} then {return}

   execute-command 5 "Mapping spectra"
   map_HM413spectra
}

proc unmap_spectra {} {
   global RDOUTEnable

   if {$RDOUTEnable == 0} then {return}

   execute-command 8 "Unmapping spectra"
   unmap_HM413spectra
}

proc map_HM413spectra {} {
    global HM413_Present 

  if {$HM413_Present == 0} then {return}

   write-register HM413.Action 5
   write-register HM413.Sig1 0
   wait_for_HM413completion
}

proc unmap_HM413spectra {} {
    global HM413_Present 

  if {$HM413_Present == 0} then {return}

   write-register HM413.Action 8
   write-register HM413.Sig1 0
   wait_for_HM413completion
}

proc wait_for_HM413completion {} {
    global RDOUTserver MaxTries

    set tries 1

    while {$tries < $MaxTries} {

        set-footer "Waiting for action to complete ($tries)"

        set z [catch {EG "register :name '/$RDOUTserver/HM413.Action' ?value"} m ]

        if {$z != 0} then {
            midas-warning "Unable to confirm state of action\n$m"
            clear-footer
            return
        }

        if {$m == 0} then {
            clear-footer
            return
        }
        incr tries
        after 500
    }

    midas-warning "Action has not completed\nGiving up waiting"
    clear-footer
}

proc click-HM413trace {v} {
    write-register HM413.Trace $v
}

proc click-HM413ID {} {
    set ID [read-register HM413.ID]
    insert-log "Target OS = [first $ID]\nProgram = [second $ID]\nVersion = [third $ID]"
}

proc click-HM413ReadOut {} {
   global CAMACserver RDOUTserver HM413_Segments
   global HM413_Present 

  if {$HM413_Present == 0} then {return}

#    check for DA going

    set camacstate [EG "register :name '/$CAMACserver/CC#1.TestI' ?value"]

#    if going stop the HM413

    if {$camacstate == "%01"} then {EG "register :pattern '/$CAMACserver/hm413#?.stop.hist' :value '0'"}

#    read out the counts

    write-register HM413.Action 1
    write-register HM413.Sig1 0
    wait_for_HM413completion

#    restart the HM413

    if {$camacstate == "%01"} then {EG "register :pattern '/$CAMACserver/hm413#?.start.hist' :value '0'"}
}
